# define basic image info dict keys
OS_FAMILY = "os_family"
OS_NAME = "os_name"
OS_ARCHI = "os_archi"
REPO_TAGS = "repo_tags"
IMG_ID = "image_id"
IMG_NAME = "image_name"
IMG_CREATION_TIME = "creation_time"

# define results parsed dict keys
TARGET_NAME = "target_name"
TARGET_CLASS = "target_class"
TARGET_TYPE = "target_type"
TARGET_VULNERABILITY_DF = "target_vul_df"

# define vulnerability dataframe column name
LIBRARY = "library"
VULNERABILITY_ID = "vulnerability_id"
VULNERABILITY_URL = "vulnerability_url"
SEVERITY = "severity"
STATUS = "status"
INSTALLED_VER = "installed_version"
FIXED_VER = "fixed_version"
DESCRIPTION = "description"
CVSS_V3_SCORE = "cvss_v3_score"
CVSS_V3_VECTOR = "cvss_v3_vector"
